@@ -1,7 +1,7 @@
-
 import logging
 from ete3 import NCBITaxa
 
+
 def compare_taxa(tax1, tax2):
     score = 0
     joined = []
@@ -11,7 +11,8 @@ def compare_taxa(tax1, tax2):
             joined.append(a)
         else:
             break
-    return(score, joined)
+    return (score, joined)
+
 
 def print_lngs(tax1, tax2):
     """
@@ -20,15 +21,15 @@ def print_lngs(tax1, tax2):
     # try guessing the line width
     try:
         import shutil
+
         s = shutil.get_terminal_size()
         linewidth = s.columns
-    except:
+    except Exception:
         linewidth = 80
-    
-    
+
     def makestr(lst):
-        return([str(i) for i in lst])
-    
+        return [str(i) for i in lst]
+
     tax1 = makestr(tax1)
     tax2 = makestr(tax2)
     score, joined = compare_taxa(tax1, tax2)
@@ -36,7 +37,7 @@ def print_lngs(tax1, tax2):
     joined = makestr(joined)
     tax1 = tax1[score:]
     tax2 = tax2[score:]
-    
+
     # lables
     label1 = "Model Lng."
     label2 = "Shared    "
@@ -46,6 +47,7 @@ def print_lngs(tax1, tax2):
     tax1r = " ".join(tax1)
     tax2r = " ".join(tax2)
     # check if a lineage or root is to long
+
     def shorten(lst):
         nw = [lst[0]]
         for e in lst[1:-3]:
@@ -53,9 +55,9 @@ def print_lngs(tax1, tax2):
                 nw.append(e)
         nw.append("...")
         nw.append(lst[-1])
-        
-        return(nw)
-    
+
+        return nw
+
     # check if we can shorten the branches
     while len(root) + len(tax1r) > linewidth or len(root) + len(tax2r) > linewidth:
         canremove = 0
@@ -68,7 +70,7 @@ def print_lngs(tax1, tax2):
                 tax1 = shorten(tax1)
         else:
             canremove = canremove - 1
-                
+
         if len(root) + len(tax2r) > linewidth and len(tax2) >= 3:
             # keep track of if there is room to shorten
             if len(tax2) <= 3 and tax2[1] == "...":
@@ -77,14 +79,14 @@ def print_lngs(tax1, tax2):
                 tax2 = shorten(tax2)
         else:
             canremove = canremove - 1
-        
+
         tax1r = " ".join(tax1)
         tax2r = " ".join(tax2)
         if canremove <= -2:
             break
-            
+
     # check if we need to shorten the root
-     # check if we can shorten the branches
+    # check if we can shorten the branches
     while len(root) + len(tax1r) > linewidth or len(root) + len(tax2r) > linewidth:
         canremove = 0
         # shorten first the branches then the root
@@ -97,7 +99,6 @@ def print_lngs(tax1, tax2):
         else:
             canremove = canremove - 1
 
-              
         root = "{} {}".format(label2, " ".join(joined))
         if len(root) + len(tax2r) > linewidth and len(joined) >= 3:
             # keep track of if there is room to shorten
@@ -109,38 +110,37 @@ def print_lngs(tax1, tax2):
             canremove = canremove - 1
 
         root = "{} {}".format(label2, " ".join(joined))
-        
+
         if canremove <= -1:
             break
-     
-                
-    
+
     space = [" "] * (len(root) - len(label1))
     space = "".join(space)
     longspace = "".join([" "] * (len(root)))
-    
-    #print dendrogram
+
+    # print dendrogram
     print("")
     print("Infered lineage compared to the model lineage:")
     if len(tax1r) > 0:
-        print(label1, end ="")
-        print(space, end = "  ")
+        print(label1, end="")
+        print(space, end="  ")
         print(tax1r)
-        print(longspace, end = "/\n")
+        print(longspace, end="/\n")
     else:
         print(label1)
         print("")
     print(root)
     if len(tax2r) > 0:
-        print(longspace, end = "\\\n")
-        print(label3, end ="")
-        print(space, end = "  ")
+        print(longspace, end="\\\n")
+        print(label3, end="")
+        print(space, end="  ")
         print(tax2r)
     else:
         print("")
         print(label3)
     print("\n")
 
+
 def write_lngs(lngs, outfile):
     """
     write infered taxonomy in a machine and human readble format
@@ -150,8 +150,8 @@ def write_lngs(lngs, outfile):
     with open(outfile, "w") as fout:
         fout.write("bin\ttaxid\tncbi_rank\tncbi_name\tbasedon\n")
         for binname, lngi in lngs.items():
-            lng = lngi['lng']
-            nprots = lngi['n']
+            lng = lngi["lng"]
+            nprots = lngi["n"]
             nms = ncbi.get_taxid_translator(lng)
             ranks = ncbi.get_rank(lng)
             for taxid in lng:
@@ -159,5 +159,5 @@ def write_lngs(lngs, outfile):
                     name = nms[taxid]
                 else:
                     name = "unnamed"
-                fout.write(f"{binname}\t{taxid}\t{ranks[taxid]}\t{name}\t{nprots}\n") 
+                fout.write(f"{binname}\t{taxid}\t{ranks[taxid]}\t{name}\t{nprots}\n")
         logging.info("Wrote lineage to %s" % outfile)
